==> flexplaylist-0.4.7.tar.bz2: flexplaylist-0.4.7/src/playlist.cpp <==
/***************************************************************************
  playlist.cpp  -  FlexPlaylists Noatun PlayLsit Object
  --------------------------------------------------------
  Maintainer: Stefan Gehn <sgehn@gmx.net>

  Copyright (c) 2000-2001 Charles Samuels <charles@kde.org>
                2000-2001 Neil Stevens <neil@qualityassistant.com>
                2003      Stefan Gehn <sgehn@gmx.net>

 ***************************************************************************/

#include "playlist.h"
#include "view.h"
#include <noatun/player.h>

#include <kapplication.h>
#include <krandomsequence.h>
#include <kdebug.h>

#include <kiconloader.h>

FlexPlaylist *FlexPlaylist::Self=0;

FlexPlaylist::FlexPlaylist()
	: Playlist(0, "FlexPlaylist"), Plugin(), mExiting(false), view(0), mPrefs(0)
{
	Self=this;
}

void FlexPlaylist::init()
{
	mPrefs = new FlexPrefs(this);
	mFullLength=0;

	view=new View("FlexPlaylistView");
	connect(view->listView(), SIGNAL(executed(QListViewItem*)), SLOT(listItemSelected(QListViewItem*)));
	connect(view, SIGNAL(shown()), SIGNAL(listShown()));
	connect(view, SIGNAL(hidden()), SIGNAL(listHidden()));
	view->init();
}

FlexPlaylist::~FlexPlaylist()
{
	kdDebug(66666) << k_funcinfo << "Called." << endl;
	mExiting=true;
	delete view;
}

void FlexPlaylist::reset()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
//	previousItem=0;
	setCurrent(static_cast<SafeListViewItem*>(view->listView()->firstChild()), false);
}

PlaylistItem FlexPlaylist::next()
{
	return next(true);
}

PlaylistItem FlexPlaylist::next(bool play)
{
	PlaylistItem nextItem;
	if (napp->player()->loopStyle() == Player::Random)
	{
		// Ignore all this order stuff and select a random item
		List *lview = view->listView();

		if (lview->childCount())
		{
			SafeListViewItem *slvi = static_cast<SafeListViewItem*>(
				lview->itemAtIndex(KApplication::random() % lview->childCount())
				);
			nextItem = PlaylistItem(slvi);
		}
		else
		{
			nextItem = 0;
		}
	}
	else
	{
		nextItem = static_cast<SafeListViewItem*>(
			static_cast<SafeListViewItem*>(current().data())->itemBelow());
	}

	if (!nextItem)
		return 0;

	PlaylistItem oldCurrent = currentItem;
	setCurrent(nextItem, play);

	// Hack for back button on randomized play
	if (oldCurrent)
	{
		randomPrevious = PlaylistItem(static_cast<SafeListViewItem*>(oldCurrent.data()));
		kdDebug(66666) << k_funcinfo << "setting randomPrevious to '" << randomPrevious.title() << "'" << endl;
		//previousItem = oldCurrent;
	}

	return currentItem;
}

PlaylistItem FlexPlaylist::current()
{
	return currentItem;
}

PlaylistItem FlexPlaylist::previous()
{
	kdDebug(66666) << k_funcinfo << "Called" << endl;
#if 0
	if (!previousItem) return 0;
	setCurrent(previousItem);

/*	if (currentItem)
		if (!static_cast<SafeListViewItem*>(currentItem.data())->isOn())
			return previous();
*/
	return currentItem;
#endif

	if (napp->player()->loopStyle() == Player::Random && randomPrevious)
	{
		kdDebug(66666) << k_funcinfo << "getting previous track for random" << endl;
		List *list = view->listView();
		// check if the item still exists (hackitude: 50%)
		bool found=false;
		for (QListViewItem *i = list->firstChild(); i; i = i->nextSibling())
		{
			if (i == static_cast<SafeListViewItem*>(randomPrevious.data()))
			{
				found = true;
				kdDebug(66666) << k_funcinfo << "found track for random, title=" << randomPrevious.title() << endl;
				break;
			}
		}

		if (found)
		{
			//setCurrentmodifiedrandomPrevious,andsetCurrentispass-by-reference
			PlaylistItem prev = randomPrevious;
			setCurrent(prev);
			return currentItem;
		}
	}
	// there's a possibility that I will fall out to here
	// from the above branch

	PlaylistItem nextItem = static_cast<SafeListViewItem*>(
			static_cast<SafeListViewItem*>(
					current().data()
				)->itemAbove()
		);
	setCurrent(nextItem);
	return currentItem;
}

PlaylistItem FlexPlaylist::getFirst() const
{
	return static_cast<SafeListViewItem*>(view->listView()->firstChild());
}

PlaylistItem FlexPlaylist::getAfter(const PlaylistItem &item) const
{
	if (item)
		return static_cast<SafeListViewItem*>(static_cast<const SafeListViewItem*>(item.data())->nextSibling());
	return 0;
}

bool FlexPlaylist::listVisible() const
{
//	kdDebug(66666) << k_funcinfo << endl;
	return view->isVisible();
}

void FlexPlaylist::showList()
{
//	kdDebug(66666) << k_funcinfo << endl;
	view->show();
	view->raise();
}

void FlexPlaylist::hideList()
{
//	kdDebug(66666) << k_funcinfo << endl;
	view->hide();
}

void FlexPlaylist::clear()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
	view->listView()->clear();
}

void FlexPlaylist::addFile(const KURL &file, bool play)
{
	view->addFile(file, play);
}

/*void FlexPlaylist::setNext(const PlaylistItem &i)
{
	nextItem=i;
}*/

void FlexPlaylist::setCurrent(const PlaylistItem &i)
{
	setCurrent(i, true);
}

void FlexPlaylist::setCurrent(const PlaylistItem &i, bool emitC)
{
	//kdDebug(66666) << k_funcinfo << "called." << endl;
	randomPrevious = PlaylistItem();
	emitC = emitC && currentItem;
	if (!i)
	{
		kdDebug(66666) << k_funcinfo << "passed PlaylistItem is NULL!" << endl;
		currentItem=0;
	}
	else
	{
		QRect rect(view->listView()->itemRect(static_cast<SafeListViewItem*>(current().data())));
		rect.setWidth(view->listView()->viewport()->width());

		// set newly selected item as current
		currentItem = i;

		view->listView()->viewport()->repaint(rect/*,true*/); // repaint previous Current item

		view->listView()->ensureItemVisible(static_cast<SafeListViewItem*>(current().data()));

		static_cast<SafeListViewItem*>(current().data())->widthChanged(TIME_COL);

		QRect currentRect = view->listView()->itemRect(static_cast<SafeListViewItem*>(current().data()));
		view->listView()->viewport()->repaint(currentRect); // repaint new Current item

		//setNext(static_cast<SafeListViewItem*>(static_cast<SafeListViewItem*>(current().data())->itemBelow()));
		//setPrevious(static_cast<SafeListViewItem*>(static_cast<SafeListViewItem*>(current().data())->itemAbove()));
	}

//	view->updateStatusBar();

	if (emitC && !exiting())
		emit playCurrent();
}

/*void FlexPlaylist::setPrevious(const PlaylistItem &i)
{
	previousItem=i;
}*/

void FlexPlaylist::remove(const PlaylistItem &)
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
//	delete i;
}

void FlexPlaylist::listItemSelected(QListViewItem *i)
{
	setCurrent(PlaylistItem(static_cast<SafeListViewItem*>(i)), false);
	emit playCurrent();
}

void FlexPlaylist::randomize()
{
	// turning off sorting is necessary
	// otherwise, the list will get randomized and promptly sorted again
	view->setSorting(false);
	List *lview = view->listView();
	// eeeeevil :)
	QPtrList<void> list;
	QPtrList<QListViewItem> items;
	for ( int i = 0; i < lview->childCount(); i++ )
	{
		list.append( (void*) i );
		items.append( lview->itemAtIndex( i ) );
	}

	KRandomSequence seq;
	seq.randomize( &list );

	for(int i = 0; i < lview->childCount(); i++)
	{
		//lview->moveItem(items.take(), 0, lview->itemAtIndex((int) list.take()));
		items.take()->moveItem(lview->itemAtIndex((int) list.take()));
	}

	setCurrent(currentItem, false);
}

void FlexPlaylist::sort()
{
	view->setSorting(true);
	setCurrent(currentItem, false);
}

void FlexPlaylist::lengthChanged(QString &oldTime, QString &newTime)
{
	unsigned long len1 = stringToLen(oldTime);
	unsigned long len2 = stringToLen(newTime);

	mFullLength += (len2-len1);

	if ((len2-len1) != 0)
	{
//		kdDebug(66666) << "oldlen=" << len1 << ", newlen=" << len2 <<
// 			", mFullLength is now " << mFullLength << endl;
		view->updateStatusBar();
	}
}

unsigned long FlexPlaylist::stringToLen(QString &s)
{
	int dotpos = s.find(':');
	unsigned long sec = (s.mid(dotpos+1)).toULong();
	unsigned long min = (s.left(dotpos)).toULong();
	unsigned long res =  (min*60) + sec;
	return res;
}

/*
void FlexPlaylist::added(PlaylistItem &i)
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
}
*/

void FlexPlaylist::removed(PlaylistItem &i)
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
	mFullLength -= i.length()/1000;
	view->updateStatusBar();
}

/*
void FlexPlaylist::modified(PlaylistItem &i)
{
	kdDebug(66666) << k_funcinfo << "Called." << endl;
	emit itemModified(i);
}
*/
#include "playlist.moc"

==> flexplaylist-0.4.8.tar.bz2: flexplaylist-0.4.8/src/playlist.cpp <==
/***************************************************************************
  playlist.cpp  -  FlexPlaylists Noatun PlayLsit Object
  --------------------------------------------------------
  Maintainer: Stefan Gehn <sgehn@gmx.net>

  Copyright (c) 2000-2001 Charles Samuels <charles@kde.org>
                2000-2001 Neil Stevens <neil@qualityassistant.com>
                2003      Stefan Gehn <sgehn@gmx.net>

 ***************************************************************************/

#include "playlist.h"
#include "view.h"
#include <noatun/player.h>

#include <kapplication.h>
#include <krandomsequence.h>
#include <kdebug.h>

#include <kiconloader.h>

FlexPlaylist *FlexPlaylist::Self=0;

FlexPlaylist::FlexPlaylist()
	: Playlist(0, "FlexPlaylist"), Plugin(), mExiting(false), mUpdating(false), view(0), mPrefs(0)
{
	//kdDebug(66666) << k_funcinfo << "Called." << endl;
	Self=this;
}

void FlexPlaylist::init()
{
	mPrefs = new FlexPrefs(this);
	mFullLength=0;

	view=new View("FlexPlaylistView");
	connect(view->listView(), SIGNAL(executed(QListViewItem*)), SLOT(listItemSelected(QListViewItem*)));
	connect(view, SIGNAL(shown()), SIGNAL(listShown()));
	connect(view, SIGNAL(hidden()), SIGNAL(listHidden()));
	view->init();
}

FlexPlaylist::~FlexPlaylist()
{
	//kdDebug(66666) << k_funcinfo << "Called." << endl;
	mExiting=true;
	delete view;
}

void FlexPlaylist::reset()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
	setCurrent(static_cast<SafeListViewItem*>(view->listView()->firstChild()), false);
}

PlaylistItem FlexPlaylist::next()
{
	return next(true);
}

PlaylistItem FlexPlaylist::next(bool play)
{
	//kdDebug(66666) << k_funcinfo << "Called" << endl;

	PlaylistItem nextItem;
	if (napp->player()->loopStyle() == Player::Random)
	{
		// Ignore all this order stuff and select a random item
		FPListView *lview = view->listView();

		if (lview->childCount())
		{
			// TODO: find a better way to handle random playing, this is not very randomized
			SafeListViewItem *slvi = static_cast<SafeListViewItem*>(
				lview->itemAtIndex(KApplication::random() % lview->childCount())
				);
			nextItem = PlaylistItem(slvi);
		}
		else
		{
			nextItem = 0;
		}
	}
	else
	{
		if(!current())
		{
			nextItem = static_cast<SafeListViewItem*>(static_cast<SafeListViewItem*>(getFirst().data()));
		}
		else
		{
			nextItem = static_cast<SafeListViewItem*>(
				static_cast<SafeListViewItem*>(current().data())->itemBelow());
		}
	}

	if (!nextItem) // don't set a null-item as current item
		return 0;

	PlaylistItem oldCurrent = currentItem;
	setCurrent(nextItem, play);

	// Hack for back button on randomized play
	if (oldCurrent)
		randomPrevious = PlaylistItem(static_cast<SafeListViewItem*>(oldCurrent.data()));

	return currentItem;
}

PlaylistItem FlexPlaylist::current()
{
	return currentItem;
}

PlaylistItem FlexPlaylist::previous()
{
	//kdDebug(66666) << k_funcinfo << "Called" << endl;

	if (napp->player()->loopStyle() == Player::Random && randomPrevious)
	{
		//kdDebug(66666) << k_funcinfo << "getting previous track for random" << endl;
		// check if the item still exists (hackitude: 50%)
		bool found=false;
		for (QListViewItem *i = view->listView()->firstChild(); i; i = i->nextSibling())
		{
			if (i == static_cast<SafeListViewItem*>(randomPrevious.data()))
			{
				found = true;
				//kdDebug(66666) << k_funcinfo << "found track for random, title=" << randomPrevious.title() << endl;
				break;
			}
		}

		if (found)
		{
			//setCurrentmodifiedrandomPrevious,andsetCurrentispass-by-reference
			PlaylistItem prev = randomPrevious;
			setCurrent(prev);
			return currentItem;
		}
	}
	// there's a possibility that I will fall out to here
	// from the above branch

	PlaylistItem nextItem;
	if(!current())
	{
		nextItem = static_cast<SafeListViewItem*>(static_cast<SafeListViewItem*>(getFirst().data()));
	}
	else
	{
		nextItem = static_cast<SafeListViewItem*>(
			static_cast<SafeListViewItem*>(current().data())->itemAbove());
	}

	if (!nextItem) // don't set a null-item as current item
		return 0;

	setCurrent(nextItem);
	return currentItem;
}

PlaylistItem FlexPlaylist::getFirst() const
{
	return static_cast<SafeListViewItem*>(view->listView()->firstChild());
}

PlaylistItem FlexPlaylist::getAfter(const PlaylistItem &item) const
{
	if (item)
		return static_cast<SafeListViewItem*>(static_cast<const SafeListViewItem*>(item.data())->nextSibling());
	return 0;
}

bool FlexPlaylist::listVisible() const
{
//	kdDebug(66666) << k_funcinfo << endl;
	return view->isVisible();
}

void FlexPlaylist::showList()
{
//	kdDebug(66666) << k_funcinfo << endl;
	view->show();
	view->raise();
}

void FlexPlaylist::hideList()
{
//	kdDebug(66666) << k_funcinfo << endl;
	view->hide();
}

void FlexPlaylist::clear()
{
	kdDebug(66666) << k_funcinfo << "Called." << endl;
	view->listView()->clear();
}

void FlexPlaylist::updating(bool b)
{
	//kdDebug(66666) << k_funcinfo << "Called, updateflag=" << b << endl;
	mUpdating = b;
	if (!mUpdating) // end of longer list update, redo statusbar
		view->updateStatusBar();
}
bool FlexPlaylist::updating()
{
	return mUpdating;
}

void FlexPlaylist::addFile(const KURL &file, bool play)
{
	view->addFile(file, play);
}

void FlexPlaylist::setCurrent(const PlaylistItem &i)
{
	setCurrent(i, true);
}

void FlexPlaylist::setCurrent(const PlaylistItem &i, bool emitC)
{
	//kdDebug(66666) << k_funcinfo << "called." << endl;
	randomPrevious = PlaylistItem();
	emitC = emitC && currentItem;
	if (!i)
	{
		kdDebug(66666) << k_funcinfo << "passed PlaylistItem is NULL!" << endl;
		currentItem=0;
	}
	else
	{
		QRect rect(view->listView()->itemRect(static_cast<SafeListViewItem*>(current().data())));
		rect.setWidth(view->listView()->viewport()->width());

		// set newly selected item as current
		currentItem = i;

		view->listView()->viewport()->repaint(rect); // repaint previous Current item

		SafeListViewItem *newLVI = static_cast<SafeListViewItem*>(current().data());
		view->listView()->ensureItemVisible(newLVI);
		newLVI->widthChanged(TIME_COL);
		QRect currentRect = view->listView()->itemRect(newLVI);
		view->listView()->viewport()->repaint(currentRect); // repaint new Current item
	}

	if (emitC && !mExiting)
		emit playCurrent();
}

void FlexPlaylist::remove(const PlaylistItem &)
{
	kdDebug(66666) << k_funcinfo << "Called." << endl;
}

void FlexPlaylist::listItemSelected(QListViewItem *i)
{
	setCurrent(PlaylistItem(static_cast<SafeListViewItem*>(i)), false);
	emit playCurrent();
}

void FlexPlaylist::randomize()
{
	// turning off sorting is necessary
	// otherwise, the list will get randomized and promptly sorted again
	view->setSorting(false);
	FPListView *lview = view->listView();
	// eeeeevil :)
	QPtrList<void> list;
	QPtrList<QListViewItem> items;
	for(int i = 0; i < lview->childCount(); i++)
	{
		list.append((void*) i);
		items.append(lview->itemAtIndex(i));
	}

	KRandomSequence seq;
	seq.randomize( &list );

	for(int i = 0; i < lview->childCount(); i++)
	{
		items.take()->moveItem(lview->itemAtIndex((int) list.take()));
	}

	setCurrent(currentItem, false);
}

void FlexPlaylist::sort()
{
	view->setSorting(true);
	setCurrent(currentItem, false);
}

void FlexPlaylist::lengthChanged(QString &oldTime, QString &newTime)
{
	if (mExiting)
		return;

	unsigned long lenDiff = stringToLen(newTime) - stringToLen(oldTime);
	//kdDebug(66666) << k_funcinfo << "Called, lenDiff=" << lenDiff << endl;
	mFullLength += lenDiff;
	if (!mUpdating && (lenDiff != 0)) // old and new length differ
		QTimer::singleShot(0, view, SLOT(updateStatusBar()));;
}

unsigned long FlexPlaylist::stringToLen(QString &s)
{
	int dotpos = s.find(':');
	unsigned long sec = (s.mid(dotpos+1)).toULong();
	unsigned long min = (s.left(dotpos)).toULong();
	unsigned long res = (min*60) + sec;
	return res;
}

void FlexPlaylist::removed(PlaylistItem &i)
{
	if(mExiting)
		return;

	//kdDebug(66666) << k_funcinfo << "Called." << endl;
	mFullLength -= i.length()/1000;
	if(!mUpdating)
		QTimer::singleShot(0, view, SLOT(updateStatusBar()));
}
#include "playlist.moc"
